"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
from outpost_sdk import models, utils
from outpost_sdk._hooks import SDKHooks
from outpost_sdk.destinations import Destinations
from outpost_sdk.events import Events
from outpost_sdk.health import Health
from outpost_sdk.models import internal
from outpost_sdk.publish import Publish
from outpost_sdk.schemas import Schemas
from outpost_sdk.tenants import Tenants
from outpost_sdk.topics_sdk import TopicsSDK
from outpost_sdk.types import OptionalNullable, UNSET
from typing import Callable, Dict, Optional, Union, cast
import weakref


class Outpost(BaseSDK):
    r"""Outpost API: The Outpost API is a REST-based JSON API for managing tenants, destinations, and publishing events."""

    health: Health
    r"""API Health Check"""
    tenants: Tenants
    r"""The API segments resources per `tenant`. A tenant represents a user/team/organization in your product. The provided value determines the tenant's ID, which can be any string representation.

    If your system is not multi-tenant, create a single tenant with a hard-code tenant ID upon initialization. If your system has a single tenant but multiple environments, create a tenant per environment, like `live` and `test`.

    """
    destinations: Destinations
    r"""Destinations are the endpoints where events are sent. Each destination is associated with a tenant and can be configured to receive specific event topics.

    ```json
    {
    \"id\": \"des_12345\", // Control plane generated ID or user provided ID
    \"type\": \"webhooks\", // Type of the destination
    \"topics\": [\"user.created\", \"user.updated\"], // Topics of events this destination is eligible for
    \"config\": {
    // Destination type specific configuration. Schema of depends on type
    \"url\": \"https://example.com/webhooks/user\" 
    },
    \"credentials\": {
    // Destination type specific credentials. AES encrypted. Schema depends on type
    \"secret\": \"some***********\" 
    },
    \"disabled_at\": null, // null or ISO date if disabled
    \"created_at\": \"2024-01-01T00:00:00Z\" // Date the destination was created
    }
    ```

    The `topics` array can contain either a list of topics or a wildcard `*` implying that all topics are supported. If you do not wish to implement topics for your application, you set all destination topics to `*`.

    By default all destination `credentials` are obfuscated and the values cannot be read. This does not apply to the `webhook` type destination secret and each destination can expose their own obfuscation logic.

    """
    publish: Publish
    r"""Operations for publishing events."""
    schemas: Schemas
    r"""Operations for retrieving destination type schemas."""
    topics: TopicsSDK
    r"""Operations for retrieving available event topics."""
    events: Events
    r"""Operations related to event history and deliveries."""

    def __init__(
        self,
        security: Optional[
            Union[models.Security, Callable[[], models.Security]]
        ] = None,
        tenant_id: Optional[str] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param security: The security details required for authentication
        :param tenant_id: Configures the tenant_id parameter for all supported operations
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        _globals = internal.Globals(
            tenant_id=utils.get_global_from_env(tenant_id, "TENANT_ID", str),
        )

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                globals=_globals,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

        self._init_sdks()

    def _init_sdks(self):
        self.health = Health(self.sdk_configuration)
        self.tenants = Tenants(self.sdk_configuration)
        self.destinations = Destinations(self.sdk_configuration)
        self.publish = Publish(self.sdk_configuration)
        self.schemas = Schemas(self.sdk_configuration)
        self.topics = TopicsSDK(self.sdk_configuration)
        self.events = Events(self.sdk_configuration)

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None
